/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.domain.pluginDecorators;

import java.util.Map;

import kieker.analysis.model.analysisMetaModel.MIAnalysisMetaModelFactory;
import kieker.analysis.model.analysisMetaModel.MIDisplay;
import kieker.analysis.model.analysisMetaModel.MIOutputPort;
import kieker.analysis.model.analysisMetaModel.MIPlugin;
import kieker.analysis.model.analysisMetaModel.MIRepositoryConnector;

import org.eclipse.emf.common.util.EList;

public abstract class AbstractPluginDecorator<T extends MIPlugin> extends AbstractAnalysisComponentDecorator<T> implements MIPlugin {

	private final Map<String, String> displaysDescriptions;

	public AbstractPluginDecorator(final T analysisComponent, final Map<String, String> propertiesDescriptions, final String description, final String dependency,
			final boolean fullyInitialized, final Map<String, String> displaysDescriptions) {
		super(analysisComponent, propertiesDescriptions, description, dependency, fullyInitialized);

		this.displaysDescriptions = displaysDescriptions;
	}

	@Override
	public final EList<MIRepositoryConnector> getRepositories() {
		return super.analysisComponent.getRepositories();
	}

	@Override
	public final EList<MIOutputPort> getOutputPorts() {
		return super.analysisComponent.getOutputPorts();
	}

	@Override
	public final EList<MIDisplay> getDisplays() {
		return super.analysisComponent.getDisplays();
	}

	public final String getDisplayDescription(final String display) {
		return this.displaysDescriptions.get(display);
	}

	@Override
	protected void refineComponentCopy(final MIAnalysisMetaModelFactory factory, final T componentCopy) {
		super.refineComponentCopy(factory, componentCopy);

		// Copy the output ports of the plugin instance
		for (final MIOutputPort outputPort : super.analysisComponent.getOutputPorts()) {
			final MIOutputPort outputPortCopy = factory.createOutputPort();
			outputPortCopy.setName(outputPort.getName());
			outputPortCopy.setParent(componentCopy);
			componentCopy.getOutputPorts().add(outputPortCopy);
		}

		// Copy the repository "ports"
		for (final MIRepositoryConnector repositoryConnector : super.analysisComponent.getRepositories()) {
			final MIRepositoryConnector repositoryConnectorCopy = factory.createRepositoryConnector();
			repositoryConnectorCopy.setName(repositoryConnector.getName());
			componentCopy.getRepositories().add(repositoryConnectorCopy);
		}

		// Copy the displays
		for (final MIDisplay display : super.analysisComponent.getDisplays()) {
			final MIDisplay displayCopy = factory.createDisplay();
			displayCopy.setName(display.getName());
			displayCopy.setParent(componentCopy);
			componentCopy.getDisplays().add(displayCopy);
		}
	}

}
