/***************************************************************************
 * Copyright 2013 Kieker Project (http://kieker-monitoring.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 ***************************************************************************/

package kieker.webgui.domain.pluginDecorators;

import java.util.Map;

import kieker.analysis.model.analysisMetaModel.MIAnalysisMetaModelFactory;
import kieker.analysis.model.analysisMetaModel.MIFilter;
import kieker.analysis.model.analysisMetaModel.MIInputPort;

import org.eclipse.emf.common.util.EList;

public class FilterDecorator extends AbstractPluginDecorator<MIFilter> implements MIFilter {

	public FilterDecorator(final MIFilter analysisComponent, final Map<String, String> propertiesDescriptions, final String description, final String dependency,
			final boolean fullyInitialized, final Map<String, String> displaysDescriptions) {
		super(analysisComponent, propertiesDescriptions, description, dependency, fullyInitialized, displaysDescriptions);
	}

	@Override
	public final EList<MIInputPort> getInputPorts() {
		return super.analysisComponent.getInputPorts();
	}

	@Override
	protected final MIFilter createComponent(final MIAnalysisMetaModelFactory factory) {
		return factory.createFilter();
	}

	@Override
	protected void refineComponentCopy(final MIAnalysisMetaModelFactory factory, final MIFilter componentCopy) {
		super.refineComponentCopy(factory, componentCopy);

		// Copy the input ports of the plugin instance
		for (final MIInputPort inputPort : super.analysisComponent.getInputPorts()) {
			final MIInputPort inputPortCopy = factory.createInputPort();
			inputPortCopy.setName(inputPort.getName());
			inputPortCopy.setParent(componentCopy);
			componentCopy.getInputPorts().add(inputPortCopy);
		}
	}

}
